/* jcifs smb client library in Java
 * Copyright (C) 2004  "Michael B. Allen" <jcifs at samba dot org>
 *                   "Eric Glass" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.spnego;

import java.io.IOException;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;

public class SpnegoLoginHandler implements CallbackHandler {

    private String username;

    private String password;

    public SpnegoLoginHandler() {
        this(null, null);
    }

    public SpnegoLoginHandler(String password) {
        this(null, password);
    }

    public SpnegoLoginHandler(String username, String password) {
        setUsername(username);
        setPassword(password);
    }

    public String getUsername() {
        return (username != null) ? username : (username =
                System.getProperty("javax.security.auth.login.name"));
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return (password != null) ? password : (password =
                System.getProperty("javax.security.auth.login.password"));
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void handle(Callback[] callbacks) throws IOException,
            UnsupportedCallbackException {
        for (int i = 0; i < callbacks.length; i++) {
            Callback callback = callbacks[i];
            if (callback instanceof NameCallback) {
                ((NameCallback) callback).setName(getUsername());
            } else if (callback instanceof PasswordCallback) {
                String password = getPassword();
                ((PasswordCallback) callback).setPassword(
                        (password != null) ? password.toCharArray() : null);
            }
        }
    }

}
